@@ -15,7 +15,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-search-entry.c,v 1.4 2003/11/06 14:31:38 marco Exp $
+ *  $Id: ephy-search-entry.c,v 1.5 2003/12/16 17:29:41 marco Exp $
  */
 
 #include <config.h>
@@ -100,14 +100,10 @@ ephy_search_entry_class_init (EphySearchEntryClass *klass)
 static gboolean
 ephy_search_entry_timeout_cb (EphySearchEntry *entry)
 {
-	gdk_threads_enter ();
-
 	g_signal_emit (G_OBJECT (entry), ephy_search_entry_signals[SEARCH], 0,
 		       gtk_entry_get_text (GTK_ENTRY (entry)));
 	entry->priv->timeout = 0;
 
-	gdk_threads_leave ();
-
 	return FALSE;
 }
 
