@@ -15,7 +15,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-bookmarks.c,v 1.72 2004/01/28 18:32:07 chpe Exp $
+ *  $Id: ephy-bookmarks.c,v 1.80 2004/03/01 21:32:31 chpe Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -42,7 +42,7 @@
 #include <libgnomevfs/gnome-vfs-utils.h>
 
 #define EPHY_BOOKMARKS_XML_ROOT    "ephy_bookmarks"
-#define EPHY_BOOKMARKS_XML_VERSION "1.01"
+#define EPHY_BOOKMARKS_XML_VERSION "1.03"
 #define BOOKMARKS_SAVE_DELAY (3 * 1000)
 #define MAX_FAVORITES_NUM 10
 
@@ -440,34 +440,16 @@ ephy_setup_history_notifiers (EphyBookmarks *eb)
 }
 
 static void
-bookmarks_changed_cb (EphyNode *node,
-		      EphyNode *child,
-		      EphyBookmarks *eb)
-{
-	ephy_bookmarks_save_delayed (eb, BOOKMARKS_SAVE_DELAY);
-}
-
-static void
-bookmarks_removed_cb (EphyNode *node,
-		      EphyNode *child,
-		      guint old_index,
-		      EphyBookmarks *eb)
-{
-	g_signal_emit (G_OBJECT (eb), ephy_bookmarks_signals[TREE_CHANGED], 0);
-	ephy_bookmarks_save_delayed (eb, BOOKMARKS_SAVE_DELAY);
-}
-
-static char *
-get_topics_list (EphyBookmarks *eb,
-		 EphyNode *bookmark,
-		 gboolean *no_topics)
+update_bookmark_keywords (EphyBookmarks *eb, EphyNode *bookmark)
 {
+	GValue value = { 0, };
 	GPtrArray *children;
 	int i;
 	GString *list;
+	const char *title;
+	char *normalized_keywords, *case_normalized_keywords;
 
 	list = g_string_new (NULL);
-	*no_topics = TRUE;
 
 	children = ephy_node_get_children (eb->priv->keywords);
 	for (i = 0; i < children->len; i++)
@@ -485,25 +467,77 @@ get_topics_list (EphyBookmarks *eb,
 			topic = ephy_node_get_property_string
 				(kid, EPHY_NODE_KEYWORD_PROP_NAME);
 			g_string_append (list, topic);
-			*no_topics = FALSE;
+			g_string_append (list, " ");
 		}
 	}
 
-	return g_string_free (list, FALSE);
-}
+	title = ephy_node_get_property_string
+		(bookmark, EPHY_NODE_BMK_PROP_TITLE);
+	g_string_append (list, " ");
+	g_string_append (list, title);
 
-static void
-update_topics_list (EphyNode *bookmark, const char *list)
-{
-	GValue value = { 0, };
+	normalized_keywords = g_utf8_normalize (list->str, -1, G_NORMALIZE_ALL);
+	case_normalized_keywords = g_utf8_casefold (normalized_keywords, -1);
 
 	g_value_init (&value, G_TYPE_STRING);
-	g_value_set_string (&value, list);
+	g_value_set_string (&value, case_normalized_keywords);
 	ephy_node_set_property (bookmark, EPHY_NODE_BMK_PROP_KEYWORDS,
 			        &value);
 	g_value_unset (&value);
+
+	g_string_free (list, TRUE);
+	g_free (normalized_keywords);
+	g_free (case_normalized_keywords);
 }
 
+static void
+bookmarks_changed_cb (EphyNode *node,
+		      EphyNode *child,
+		      guint property_id,
+		      EphyBookmarks *eb)
+{
+	if (property_id == EPHY_NODE_BMK_PROP_TITLE)
+	{
+		update_bookmark_keywords (eb, child);
+	}
+
+	ephy_bookmarks_save_delayed (eb, BOOKMARKS_SAVE_DELAY);
+}
+
+static void
+bookmarks_removed_cb (EphyNode *node,
+		      EphyNode *child,
+		      guint old_index,
+		      EphyBookmarks *eb)
+{
+	g_signal_emit (G_OBJECT (eb), ephy_bookmarks_signals[TREE_CHANGED], 0);
+	ephy_bookmarks_save_delayed (eb, BOOKMARKS_SAVE_DELAY);
+}
+
+static gboolean
+bookmark_is_categorized (EphyBookmarks *eb, EphyNode *bookmark)
+{
+	GPtrArray *children;
+	int i;
+
+	children = ephy_node_get_children (eb->priv->keywords);
+	for (i = 0; i < children->len; i++)
+	{
+		EphyNode *kid;
+
+		kid = g_ptr_array_index (children, i);
+
+		if (kid != eb->priv->notcategorized && 
+		    kid != eb->priv->favorites &&
+		    kid != eb->priv->bookmarks &&
+		    ephy_node_has_child (kid, bookmark))
+		{
+			return TRUE;
+		}
+	}
+
+	return FALSE;
+}
 
 static void
 topics_removed_cb (EphyNode *node,
@@ -518,22 +552,17 @@ topics_removed_cb (EphyNode *node,
 	for (i = 0; i < children->len; i++)
 	{
 		EphyNode *kid;
-		gboolean no_topics;
-		char *list;
 
 		kid = g_ptr_array_index (children, i);
-		list = get_topics_list (eb, kid, &no_topics);
 
-		if (no_topics &&
+		if (!bookmark_is_categorized (eb, kid) &&
 		    !ephy_node_has_child (eb->priv->notcategorized, kid))
 		{
 			ephy_node_add_child
 				(eb->priv->notcategorized, kid);
 		}
 
-		update_topics_list (kid, list);
-
-		g_free (list);
+		update_bookmark_keywords (eb, kid);
 	}
 
 	g_signal_emit (G_OBJECT (eb), ephy_bookmarks_signals[TREE_CHANGED], 0);
@@ -556,6 +585,39 @@ disable_bookmark_editing_notifier (GConfClient *client,
 	update_bookmark_editing (eb);
 }
 
+static void
+backup_file (const char *original_filename, const char *extension)
+{
+	char *template, *backup_filename;
+	int result = 0;
+
+	if (g_file_test (original_filename, G_FILE_TEST_EXISTS) == FALSE)
+	{
+		return;
+	}
+
+	template = g_strconcat (original_filename, ".backup-XXXXXX", NULL);
+	backup_filename = ephy_file_tmp_filename (template, extension);
+
+	if (backup_filename != NULL)
+	{
+		result = rename (original_filename, backup_filename);
+	}
+
+	if (result >= 0)
+	{
+		g_message ("Your old bookmarks file was backed up as \"%s\".\n",
+			   backup_filename);
+	}
+	else
+	{
+		g_warning ("Backup failed! Your old bookmarks file was lost.\n");
+	}
+
+	g_free (template);
+	g_free (backup_filename);
+}
+
 static void
 ephy_bookmarks_init (EphyBookmarks *eb)
 {
@@ -649,12 +711,29 @@ ephy_bookmarks_init (EphyBookmarks *eb)
 	eb->priv->smartbookmarks = ephy_node_new_with_id (db, SMARTBOOKMARKS_NODE_ID);
 	ephy_node_ref (eb->priv->smartbookmarks);
 
-	if (ephy_node_db_load_from_file (eb->priv->db, eb->priv->xml_file,
-					 EPHY_BOOKMARKS_XML_ROOT,
-					 EPHY_BOOKMARKS_XML_VERSION) == FALSE)
+	if (g_file_test (eb->priv->xml_file, G_FILE_TEST_EXISTS) == FALSE
+	    && g_file_test (eb->priv->rdf_file, G_FILE_TEST_EXISTS) == FALSE)
 	{
+		eb->priv->init_defaults = TRUE;
+	}
+	else if (ephy_node_db_load_from_file (eb->priv->db, eb->priv->xml_file,
+					      EPHY_BOOKMARKS_XML_ROOT,
+					      EPHY_BOOKMARKS_XML_VERSION) == FALSE)
+	{
+		/* save the corrupted files so the user can late try to
+		 * manually recover them. See bug #128308.
+		 */
+
+		g_warning ("Could not read bookmarks file \"%s\", trying to "
+			   "re-import bookmarks from \"%s\"\n",
+			   eb->priv->xml_file, eb->priv->rdf_file);
+
+		backup_file (eb->priv->xml_file, "xml");
+
 		if (ephy_bookmarks_import_rdf (eb, eb->priv->rdf_file) == FALSE)
 		{
+			backup_file (eb->priv->rdf_file, "rdf");
+
 			eb->priv->init_defaults = TRUE;
 		}
 	}
@@ -737,7 +816,7 @@ update_has_smart_address (EphyBookmarks *bookmarks, EphyNode *bmk, const char *a
 	{
 		if (ephy_node_has_child (smart_bmks, bmk))
 		{
-			ephy_node_add_child (smart_bmks, bmk);
+			ephy_node_remove_child (smart_bmks, bmk);
 		}
 	}
 }
@@ -767,6 +846,7 @@ ephy_bookmarks_add (EphyBookmarks *eb,
 	g_value_unset (&value);
 
 	update_has_smart_address (eb, bm, url);
+	update_bookmark_keywords (eb, bm);
 
 	ephy_node_add_child (eb->priv->bookmarks, bm);
 	ephy_node_add_child (eb->priv->notcategorized, bm);
@@ -1146,15 +1226,10 @@ ephy_bookmarks_set_keyword (EphyBookmarks *eb,
 			    EphyNode *keyword,
 			    EphyNode *bookmark)
 {
-	gboolean no_topics;
-	char *list;
-
 	if (ephy_node_has_child (keyword, bookmark)) return;
 
 	ephy_node_add_child (keyword, bookmark);
 
-	list = get_topics_list (eb, bookmark, &no_topics);
-
 	if (ephy_node_has_child (eb->priv->notcategorized, bookmark))
 	{
 		LOG ("Remove from categorized bookmarks")
@@ -1162,8 +1237,7 @@ ephy_bookmarks_set_keyword (EphyBookmarks *eb,
 			(eb->priv->notcategorized, bookmark);
 	}
 
-	update_topics_list (bookmark, list);
-	g_free (list);
+	update_bookmark_keywords (eb, bookmark);
 
 	g_signal_emit (G_OBJECT (eb), ephy_bookmarks_signals[TREE_CHANGED], 0);
 }
@@ -1173,16 +1247,11 @@ ephy_bookmarks_unset_keyword (EphyBookmarks *eb,
 			      EphyNode *keyword,
 			      EphyNode *bookmark)
 {
-	gboolean no_topics;
-	char *list;
-
 	if (!ephy_node_has_child (keyword, bookmark)) return;
 
 	ephy_node_remove_child (keyword, bookmark);
 
-	list = get_topics_list (eb, bookmark, &no_topics);
-
-	if (no_topics &&
+	if (!bookmark_is_categorized (eb, bookmark) &&
 	    !ephy_node_has_child (eb->priv->notcategorized, bookmark))
 	{
 		LOG ("Add to not categorized bookmarks")
@@ -1190,8 +1259,7 @@ ephy_bookmarks_unset_keyword (EphyBookmarks *eb,
 			(eb->priv->notcategorized, bookmark);
 	}
 
-	update_topics_list (bookmark, list);
-	g_free (list);
+	update_bookmark_keywords (eb, bookmark);
 
 	g_signal_emit (G_OBJECT (eb), ephy_bookmarks_signals[TREE_CHANGED], 0);
 }
