@@ -14,18 +14,29 @@
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *
+ *  $Id: ephy-navigation-action.c,v 1.13 2003/12/19 22:58:40 chpe Exp $
  */
 
 #include "ephy-navigation-action.h"
 #include "ephy-arrow-toolbutton.h"
 #include "ephy-window.h"
+#include "ephy-string.h"
+#include "ephy-favicon-cache.h"
+#include "ephy-history.h"
+#include "ephy-embed-shell.h"
 #include "ephy-debug.h"
 
+#include <gtk/gtkimage.h>
+#include <gtk/gtkimagemenuitem.h>
+
 static void ephy_navigation_action_init       (EphyNavigationAction *action);
 static void ephy_navigation_action_class_init (EphyNavigationActionClass *class);
 
 static GObjectClass *parent_class = NULL;
 
+#define EPHY_NAVIGATION_ACTION_GET_PRIVATE(object)(G_TYPE_INSTANCE_GET_PRIVATE ((object), EPHY_TYPE_NAVIGATION_ACTION, EphyNavigationActionPrivate))
+
 struct EphyNavigationActionPrivate
 {
 	EphyWindow *window;
@@ -44,7 +55,7 @@ ephy_navigation_action_get_type (void)
 {
 	static GType type = 0;
 
-	if (!type)
+	if (type == 0)
 	{
 		static const GTypeInfo type_info =
 		{
@@ -59,30 +70,54 @@ ephy_navigation_action_get_type (void)
 			(GInstanceInitFunc) ephy_navigation_action_init,
 		};
 
-		type = g_type_register_static (EGG_TYPE_ACTION,
+		type = g_type_register_static (GTK_TYPE_ACTION,
 					       "EphyNavigationAction",
 					       &type_info, 0);
 	}
+
 	return type;
 }
 
+#define MAX_LENGTH 60
+
 static GtkWidget *
-new_history_menu_item (gchar *origtext,
-                       const GdkPixbuf *ico)
+new_history_menu_item (const char *origtext,
+		       const char *address)
 {
-        GtkWidget *item = gtk_image_menu_item_new ();
-        GtkWidget *hb = gtk_hbox_new (FALSE, 0);
-        GtkWidget *label = gtk_label_new (origtext);
+	GtkWidget *item, *image;
+	GdkPixbuf *icon = NULL;
+	char *short_text;
 
-        gtk_box_pack_start (GTK_BOX (hb), label, FALSE, FALSE, 0);
-        gtk_container_add (GTK_CONTAINER (item), hb);
+	if (address != NULL)
+	{
+		EphyFaviconCache *cache;
+		EphyHistory *history;
+		const char *icon_address;
 
-        gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (item),
-                                       gtk_image_new_from_pixbuf ((GdkPixbuf *) ico));
+		history = EPHY_HISTORY
+			(ephy_embed_shell_get_global_history (embed_shell));
+		icon_address = ephy_history_get_icon (history, address);
 
-        gtk_widget_show_all (item);
+		cache = EPHY_FAVICON_CACHE
+			(ephy_embed_shell_get_favicon_cache (embed_shell));
+		icon = ephy_favicon_cache_get (cache, icon_address);
+	}
 
-        return item;
+	short_text = ephy_string_shorten (origtext, MAX_LENGTH);
+	item = gtk_image_menu_item_new_with_label (short_text);
+	g_free (short_text);
+
+	image = gtk_image_new_from_pixbuf (icon);
+	gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (item), image);
+	gtk_widget_show (image);
+	gtk_widget_show (item);
+
+	if (icon != NULL)
+	{
+		g_object_unref (icon);
+	}
+
+	return item;
 }
 
 static void
@@ -112,7 +147,7 @@ activate_up_menu_item_cb (GtkWidget *menu, EphyWindow *window)
 
 	go_nth = GPOINTER_TO_INT(g_object_get_data (G_OBJECT(menu), "go_nth"));
 
-	ephy_embed_get_go_up_list (embed, &l);
+	l = ephy_embed_get_go_up_list (embed);
 
 	url = g_slist_nth_data (l, go_nth);
 	if (url)
@@ -134,8 +169,8 @@ setup_back_or_forward_menu (EphyWindow *window, GtkMenuShell *ms, EphyNavigation
 	embed = ephy_window_get_active_embed (window);
 	g_return_if_fail (embed != NULL);
 
-	ephy_embed_shistory_get_pos (embed, &pos);
-	ephy_embed_shistory_count (embed, &count);
+	pos = ephy_embed_shistory_get_pos (embed);
+	count = ephy_embed_shistory_n_items (embed);
 
 	if (count == 0) return;
 
@@ -155,7 +190,7 @@ setup_back_or_forward_menu (EphyWindow *window, GtkMenuShell *ms, EphyNavigation
 		char *title, *url;
 		GtkWidget *item;
 		ephy_embed_shistory_get_nth (embed, start, FALSE, &url, &title);
-		item = new_history_menu_item (title ? title : url, NULL);
+		item = new_history_menu_item (title ? title : url, url);
 		gtk_menu_shell_append (ms, item);
 		g_object_set_data (G_OBJECT (item), "go_nth", GINT_TO_POINTER (start));
 		g_signal_connect (item, "activate",
@@ -187,14 +222,14 @@ setup_up_menu (EphyWindow *window, GtkMenuShell *ms)
 	embed = ephy_window_get_active_embed (window);
 	g_return_if_fail (embed != NULL);
 
-	ephy_embed_get_go_up_list (embed, &l);
+	l = ephy_embed_get_go_up_list (embed);
 
 	for (li = l; li; li = li->next)
 	{
 		char *url = li->data;
 		GtkWidget *item;
 
-		item = new_history_menu_item (url, NULL);
+		item = new_history_menu_item (url, url);
 		gtk_menu_shell_append (ms, item);
 		g_object_set_data (G_OBJECT(item), "go_nth", GINT_TO_POINTER (count));
 		g_signal_connect (item, "activate",
@@ -241,14 +276,14 @@ menu_activated_cb (EphyArrowToolButton *w, EphyNavigationAction *b)
 }
 
 static void
-connect_proxy (EggAction *action, GtkWidget *proxy)
+connect_proxy (GtkAction *action, GtkWidget *proxy)
 {
 	LOG ("Connect navigation action proxy")
 
 	g_signal_connect (proxy, "menu-activated",
 			  G_CALLBACK (menu_activated_cb), action);
 
-	(* EGG_ACTION_CLASS (parent_class)->connect_proxy) (action, proxy);
+	(* GTK_ACTION_CLASS (parent_class)->connect_proxy) (action, proxy);
 }
 
 static void
@@ -293,32 +328,18 @@ ephy_navigation_action_get_property (GObject *object,
 	}
 }
 
-static void
-ephy_navigation_action_finalize (GObject *object)
-{
-	EphyNavigationAction *action = EPHY_NAVIGATION_ACTION (object);
-
-	g_return_if_fail (action->priv != NULL);
-
-	g_free (action->priv);
-
-	G_OBJECT_CLASS (parent_class)->finalize (object);
-}
-
 static void
 ephy_navigation_action_class_init (EphyNavigationActionClass *class)
 {
-	EggActionClass *action_class;
 	GObjectClass *object_class = G_OBJECT_CLASS (class);
+	GtkActionClass *action_class = GTK_ACTION_CLASS (class);
 
-	object_class->finalize = ephy_navigation_action_finalize;
 	object_class->set_property = ephy_navigation_action_set_property;
 	object_class->get_property = ephy_navigation_action_get_property;
 
 	parent_class = g_type_class_peek_parent (class);
-	action_class = EGG_ACTION_CLASS (class);
 
-	action_class->toolbar_item_type = EPHY_ARROW_TOOLBUTTON_TYPE;
+	action_class->toolbar_item_type = EPHY_TYPE_ARROW_TOOLBUTTON;
 	action_class->connect_proxy = connect_proxy;
 
 	g_object_class_install_property (object_class,
@@ -338,12 +359,13 @@ ephy_navigation_action_class_init (EphyNavigationActionClass *class)
                                                               G_TYPE_OBJECT,
                                                               G_PARAM_READWRITE));
 
+	g_type_class_add_private (object_class, sizeof(EphyNavigationActionPrivate));
 }
 
 static void
 ephy_navigation_action_init (EphyNavigationAction *action)
 {
-        action->priv = g_new0 (EphyNavigationActionPrivate, 1);
+        action->priv = EPHY_NAVIGATION_ACTION_GET_PRIVATE (action);
 }
 
 
